#!/usr/bin/python
import os
import sys
import json
import upload
import argparse
import fl3

def print_help():
    print ('*' * 80)
    print (' ' * 24 + 'Flamingo CLI')
    print ('*' * 80)
    print (' Available commands')
    print ('-' * 80)
    print ('  upload\t\t\t%s' % upload.summary)
    print ('*' * 80)

    print ('examples)')
    for usage in upload.get_usage_examples():
        print ('\t' + usage)

def login(args):
    print 'login... user=%s pw=%s domain=%s' % (args.user, args.pw, args.domain)
    auth = fl3.Auth()
    auth.login(args.domain, args.user, args.pw)
    auth.save_cookie()

    if auth.validate():
        print 'Success'
    else:
        exit(1)

def upload(args):
    print 'uploading... source=%s dest=%s permission=%s own=%s group=%s' % (args.source, args.dest, args.permission, args.own, args.group)
    auth = fl3.Auth()
    auth.load_cookie()

    file = fl3.File(auth)
    r = file.upload_file(args.source, args.dest, True)

    if args.own or args.group or args.permission:
        file.permission(args.dest, owner=args.own, group=args.group, permission=args.permission)

    if r:
        print 'Success'
    else:
        exit(1)

def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    # login
    parser_hdfs = subparsers.add_parser('login')
    parser_hdfs.add_argument('-user', help="Flamingo WEB username", required=True)
    parser_hdfs.add_argument('-pw', help="Flamingo WEB password", required=True)
    parser_hdfs.add_argument('-domain', help="Flamingo WEB domain", required=True)
    parser_hdfs.set_defaults(func=login)

    # upload
    parser_hdfs = subparsers.add_parser('upload')
    parser_hdfs.add_argument('-source', help="local filename (source)", required=True)
    parser_hdfs.add_argument('-dest', help="HDFS filename (destination)", required=True)
    parser_hdfs.add_argument('-permission', help="Permission (644|755)", default="644")
    parser_hdfs.add_argument('-own', help="Owner username 'hdfs", default="hdfs")
    parser_hdfs.add_argument('-group', help="Group name 'hdfs", default="hdfs")
    parser_hdfs.set_defaults(func=upload)

    parser.parse_args()
    args = parser.parse_args(sys.argv[1:])
    args.func(args)

if __name__ == '__main__':
    main()


import os
import fl3
import sys
import argparse

summary = 'Upload file to HDFS (using Flamingo REST API)'
desc = '''
    %s
    $ flcli upload username:password@domain [local filename] [dest filename]
''' % summary

def print_help():
    print ('*' * 80)
    print (' ' * 25 + 'upload')
    print ('*' * 80)
    print (desc)
    print ('*' * 80)
    print ('*' * 80)

    print ('ex)')
    for usage in get_usage_examples():
        print ('\t' + usage)

def get_usage_examples():
    return [
        '$ flcli upload exem:1234@flamingo-exem.org workflow.xml /flamingo/oozie/test-wf/workflow.xml'
    ]

def main_upload(argv):
    sys.argv = argv

    parser = argparse.ArgumentParser()
    parser.add_argument("-username", help="user name")
    parser.add_argument("-password", help="input password")
    parser.add_argument("-domain", help="domain")
    parser.add_argument("-source", help="Local filename")
    parser.add_argument("-dest", help="HDFS filename")

    parser.add_argument("-own", help="Owner")
    parser.add_argument("-group", help="Group")

    args = parser.parse_args()

    print args

    # if len(argv) >= 4:
    #     connect_info = argv[2]
    #     local_filename = argv[3]
    #     dest_filename = argv[4]
    #
    #     username = connect_info.split(':')[0]
    #     password = connect_info.split(':')[1].split('@')[0]
    #     domain = connect_info.split('@')[1]
    #
    #     # verify local file
    #     if os.path.isfile(local_filename) == False:
    #         print "Not exists file '%s'" % local_filename
    #         exit(1)
    #
    #     f = fl3.File(domain)
    #     f.login(username, password)
    #     r = f.upload_file(local_filename, dest_filename, True)
    #
    #     if r:
    #         print "Upload Success. Local file '%s' -> HDFS file '%s' " % (local_filename, dest_filename)
    #
    # else:
    #   print_help()




import urllib
import urllib2
import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder
import json
import re
import os
import time

'''
f = fl3.File(domain='flamingo.exem-oss.org')
f.login('user', 'pw')
f.delete('/user/hive/password/README.md')
f.upload_file('README.md', '/user/hive/password/README.md')

# TO BE
f.chmod('/user/hive/password/README.md', '755')
f.chown('/user/hive/password/README.md', 'hdfs:hdfs')
'''

COOKIE_FILE = os.path.join(str(os.getenv("HOME")), '.fladm_cookie.json')

def build_headers(raw):
    headers = {}

    for line in raw.split('\n'):
        line = line.strip()
        if len(line) > 0:
            t = line.lstrip().split(':')
            headers[t[0]] = t[1].lstrip()

    return headers

def get_filename(filename):
    return filename.split('/')[-1]


class Auth:
    def __init__(self):
        pass

    domain = 'flamingo.exem-oss.org'
    cookie = None
    expired = None

    def exits_cookie(self):
        return os.path.exists(COOKIE_FILE)

    def validate(self):
        if self.exits_cookie():
            self.load_cookie()
            now = time.time()

            if now > self.expired:
                return False

        return bool(re.match('JSESSIONID=[a-zA-Z_0-9]{32}', self.cookie))

    def save_cookie(self):
        f = open(COOKIE_FILE, 'w')
        f.write(json.dumps({
            'domain': self.domain,
            'cookie': self.cookie,
            'expired': time.time() + 600
        }))
        f.close()

    def load_cookie(self):
        f = open(COOKIE_FILE, 'r')
        o = json.loads(f.read())
        self.domain = o['domain']
        self.cookie = o['cookie']
        self.expired = o['expired']

    def login(self, domain, username, password):
        self.domain = domain

        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
            # 'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            # 'Content-Length': '28',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Host': 'flamingo.exem-oss.org',
            'Origin': 'http://flamingo.exem-oss.org',
            'Pragma': 'no-cache',
            'Referer': 'http://flamingo.exem-oss.org/login',
            # 'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36'
        }

        url = 'http://%s/login' % self.domain

        data = urllib.urlencode({
            'username': username,
            'password': password
        })

        s = requests.session()
        s.headers.update(headers)

        s.get(url)
        r = s.post(url, data=data, allow_redirects=False)

        cookie = 'JSESSIONID=%s' % r.cookies.get('JSESSIONID')

        # set cookie to server
        headers['Cookie'] = cookie
        s.headers.update(headers)
        s.get(url)

        self.cookie = cookie
        return cookie

class File:
    auth = None
    domain = None
    cookie = None

    def __init__(self, auth):
        self.auth = auth
        self.domain = auth.domain
        self.cookie = auth.cookie

    def get_headers(self):
        header_raw = '''
            Host: flamingo.exem-oss.org
            Connection: keep-alive
            Content-Length: 19542
            X-CSRF-Token:
            Origin: http://%s
            X-Requested-With: XMLHttpRequest
            User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36
            Content-Type: application/json; charset=UTF-8;
            Accept: */*
            Referer: http://flamingo.exem-oss.org/
            Accept-Encoding: gzip, deflate
            Accept-Language: ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7
            Cookie: %s
        ''' % (self.domain, self.cookie)

        return build_headers(header_raw)

    def delete(self, filename):
        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/delete.json' % self.domain

        data = json.dumps({
            'currentPath': '/',
            'deleteList': [{
                'fullyQualifiedPath': filename,
                'directory': False
            }]
        })

        r = requests.post(url, data=data, headers=headers)
        return json.loads(r.content)['success']

    def permission(self, filename, group="hdfs", owner="hdfs", permission="644"):
        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/setPermission' % self.domain

        data = json.dumps({
            'currentPath': filename,
            'fileNames': get_filename(filename),
            'fileStatus': 'FILE',
            'files': filename,
            'group': group,
            'owner': owner,
            'permission': permission,
            'recursiveOwner': 0,
            'recursivePermission': 0
        })

        r = requests.post(url, data=data, headers=headers)
        return json.loads(r.content)['success']

    def upload_file(self, filename, dest_filename, overwrite=False):
        if overwrite:
            self.delete(dest_filename)

        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/upload.json' % self.domain

        dstPath = '/'.join(dest_filename.split('/')[0:-1])
        fileName = get_filename(dest_filename)

        form_data = MultipartEncoder(fields={
            'dstPath': dstPath,
            'fileName': (fileName, open(filename, 'rb'))
        })

        headers['Content-Type'] = form_data.content_type

        r = requests.post(url, data=form_data, headers=headers)

        return json.loads(r.content)['success']
